﻿#include "widget.h"
#include "ui_widget.h"

#include "lymark.h"

#include <QTimer>
#include <QDir>
#include <QFile>
#include <QCoreApplication>
#include <QMessageBox>
#include <QFileDialog>
#include <QStringListModel>

static Widget *_s_wgtInstance = Q_NULLPTR;

static void on_mark_finish(int finishedCount,double time)
{
    if(_s_wgtInstance) {
        _s_wgtInstance->updateMarkFinishInfo(finishedCount,time);
    }
}

static void on_mark_status(int status)
{
    if(_s_wgtInstance) {
        _s_wgtInstance->updateMarkStatus(status);
    }
}

Widget::Widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Widget)
{
    ui->setupUi(this);

    _p_library = new QLibrary(this);

    _s_wgtInstance = this;

    QTimer::singleShot(10,this,SLOT(onLoadLibrary()));
}

Widget::~Widget()
{
    _s_wgtInstance = Q_NULLPTR;

    f_lm_destruct func = (f_lm_destruct)_p_library->resolve("lm_destruct");
    if(func) {
        func();
    }

    delete ui;
}

void Widget::updateMarkStatus(int status)
{
    if(status == 0) {
        ui->markStatusLbl->setText(tr("Stop"));
        ui->markStatusLbl->setStyleSheet("color:black;");
    } else if(status == 1) {
        ui->markStatusLbl->setText(tr("Marking"));
        ui->markStatusLbl->setStyleSheet("color:red;");
    } else if(status == 2) {
        ui->markStatusLbl->setText(tr("Previewing"));
        ui->markStatusLbl->setStyleSheet("color:red;");
    }
}

void Widget::updateMarkFinishInfo(int finishedCount,double time)
{
    ui->finishedCountSpx->setValue(finishedCount);
    ui->finishedTimeLbl->setText(QString::number(time,'f',2));
}

void Widget::onLoadLibrary()
{
    QString dllPath = QDir(QCoreApplication::applicationDirPath()).filePath("LYMark.dll");
    if(!QFile::exists(dllPath)) {
        QMessageBox::information(this,tr("Error"),tr("No found library!"),QMessageBox::Close);
        QTimer::singleShot(10,qApp,SLOT(quit()));
        return;
    }

    _p_library->setFileName(dllPath);

    if(!_p_library->load()) {
        QMessageBox::information(this,tr("Error"),tr("Load library failed!"),QMessageBox::Close);
        QTimer::singleShot(10,qApp,SLOT(quit()));
        return;
    }
}

void Widget::on_initBtn_clicked()
{
    f_lm_init func = (f_lm_init)_p_library->resolve("lm_init");

    int ret = -1;
    if(func) {
        ret = func();
    }

    if(ret == -1) {
        QMessageBox::information(this,tr("Error"),tr("Call initial control board function error."),QMessageBox::Close);
        return;
    }

    if(ret > 0) {
        QMessageBox::information(this,tr("Error"),tr("Initial control board error,code %1.").arg(ret),QMessageBox::Close);
        return;
    }

    if(ret == 0) {
        f_lm_set_mark_finish_func setCallbackFunc = (f_lm_set_mark_finish_func)_p_library->resolve("lm_set_mark_finish_func");
        if(setCallbackFunc) {
            setCallbackFunc(on_mark_finish);
        }

        f_lm_set_mark_status_func statusCallbackFunc = (f_lm_set_mark_status_func)_p_library->resolve("lm_set_mark_status_func");
        if(statusCallbackFunc) {
            statusCallbackFunc(on_mark_status);
        }

        f_lm_read_board_id idFunc = (f_lm_read_board_id)_p_library->resolve("lm_read_board_id");
        if(idFunc) {
            QByteArray idBytes;
            idFunc(idBytes);

            if(!idBytes.isEmpty()) {
                ui->idLbl->setText(idBytes);
            }
        }

        ui->selDocBtn->setEnabled(true);
        ui->entityListView->setEnabled(true);
        ui->selEntityTextEdit->setEnabled(true);
        ui->selEntityTextBtn->setEnabled(true);
        ui->selEntitySetTextEdit->setEnabled(true);
        ui->selEntityTextEditBtn->setEnabled(true);
        ui->finishedCountSpx->setEnabled(true);
        ui->startMarkBtn->setEnabled(true);
        ui->previewBtn->setEnabled(true);
        ui->stopBtn->setEnabled(true);
        ui->selEntityMarkBtn->setEnabled(true);
        ui->offsetX1Spx->setEnabled(true);
        ui->offsetY1Spx->setEnabled(true);
        ui->angle1Spx->setEnabled(true);
        ui->offsetX2Spx->setEnabled(true);
        ui->offsetY2Spx->setEnabled(true);
        ui->angle2Spx->setEnabled(true);
        ui->transformMarkBtn->setEnabled(true);
    }
}

void Widget::on_selDocBtn_clicked()
{
    QString path = QFileDialog::getOpenFileName(this,tr("Select"),QDir::currentPath(),QString("*.lmf4"));
    if(path.isEmpty()) {
        return;
    }

    f_lm_openDocument func = (f_lm_openDocument)_p_library->resolve("lm_openDocument");
    int ret = 0;
    if(func) {
        ret = func(path);
    }

    if(ret == 1) {
        QMessageBox::information(this,tr("Error"),tr("Document path can no be empty."),QMessageBox::Close);
        return;
    }

    if(ret == 2) {
        QMessageBox::information(this,tr("Error"),tr("Document can not be found."),QMessageBox::Close);
        return;
    }

    if(ret == 3) {
        QMessageBox::information(this,tr("Error"),tr("Document open failed."),QMessageBox::Close);
        return;
    }

    ui->docNameLbl->setText(QFileInfo(path).fileName());

    _entityNames.clear();
    f_lm_get_entities entityFunc = (f_lm_get_entities)_p_library->resolve("lm_get_entities");
    if(entityFunc) {
        entityFunc(_entityNames);
        ui->entityCountLbl->setText(QString::number(_entityNames.size()));
    }

    ui->entityListView->setModel(new QStringListModel(_entityNames,ui->entityListView));
}

void Widget::on_selEntityTextBtn_clicked()
{
    int selRow = ui->entityListView->currentIndex().row();
    if((selRow < 0) || (selRow >= _entityNames.size())) {
        QMessageBox::information(this,tr("Error"),tr("Please select entity first."),QMessageBox::Close);
        return;
    }

    QString entityName = _entityNames.at(selRow);
    f_lm_get_text func = (f_lm_get_text)_p_library->resolve("lm_get_text");
    if(func) {
        QString text;
        func(entityName,text);
        ui->selEntityTextEdit->setPlainText(text);
    }
}

void Widget::on_selEntityTextEditBtn_clicked()
{
    int selRow = ui->entityListView->currentIndex().row();
    if((selRow < 0) || (selRow >= _entityNames.size())) {
        QMessageBox::information(this,tr("Error"),tr("Please select entity first."),QMessageBox::Close);
        return;
    }

    QString text = ui->selEntitySetTextEdit->toPlainText();
    if(text.isEmpty()) {
        QMessageBox::information(this,tr("Error"),tr("Please input text first."),QMessageBox::Close);
        return;
    }

    QString entityName = _entityNames.at(selRow);
    f_lm_set_text func = (f_lm_set_text)_p_library->resolve("lm_set_text");
    bool ret = false;
    if(func) {
        ret = func(entityName,text);
    }

    if(!ret) {
        QMessageBox::information(this,tr("Error"),tr("Modify entity text failed."),QMessageBox::Close);
    } else {
        QMessageBox::information(this,tr("Error"),tr("Modify entity text ok."),QMessageBox::Close);
    }
}

void Widget::on_startMarkBtn_clicked()
{
    int errCode = MARK_NO_ERROR;
    f_lm_startMark func = (f_lm_startMark)_p_library->resolve("lm_startMark");
    if(func) {
        errCode = func();
    }

    if(errCode != MARK_NO_ERROR) {
        QMessageBox::information(this,tr("Error"),tr("Start mark error(%1).").arg(errCode),QMessageBox::Close);
    }
}

void Widget::on_previewBtn_clicked()
{
    int errCode = MARK_NO_ERROR;
    f_lm_startInfraredPreview func = (f_lm_startInfraredPreview)_p_library->resolve("lm_startInfraredPreview");
    if(func) {
        errCode = func();
    }

    if(errCode != MARK_NO_ERROR) {
        QMessageBox::information(this,tr("Error"),tr("Start infrared preview error(%1).").arg(errCode),QMessageBox::Close);
    }
}

void Widget::on_stopBtn_clicked()
{
    bool ret = false;
    f_lm_stopPrint func = (f_lm_stopPrint)_p_library->resolve("lm_stopPrint");
    if(func) {
        ret = func();
    }

    if(!ret) {
        QMessageBox::information(this,tr("Error"),tr("Stop failed."),QMessageBox::Close);
    }
}

void Widget::on_selEntityMarkBtn_clicked()
{
    int selRow = ui->entityListView->currentIndex().row();
    if((selRow < 0) || (selRow >= _entityNames.size())) {
        QMessageBox::information(this,tr("Error"),tr("Please select entity first."),QMessageBox::Close);
        return;
    }

    QString entityName = _entityNames.at(selRow);
    f_lm_startMarkEntity func = (f_lm_startMarkEntity)_p_library->resolve("lm_startMarkEntity");
    int errCode = MARK_NO_ERROR;
    if(func) {
        errCode = func(entityName);
    }

    if(errCode != MARK_NO_ERROR) {
        QMessageBox::information(this,tr("Error"),tr("Start mark error(%1).").arg(errCode),QMessageBox::Close);
    }
}

void Widget::on_transformMarkBtn_clicked()
{
    int errCode = MARK_NO_ERROR;
    f_lm_startTransformMarkDoc func = (f_lm_startTransformMarkDoc)_p_library->resolve("lm_startTransformMarkDoc");
    if(func) {
        QList<double> offsetXs,offsetYs,angles;
        offsetXs << ui->offsetX1Spx->value() << ui->offsetX2Spx->value();
        offsetYs << ui->offsetY1Spx->value() << ui->offsetY2Spx->value();
        angles << ui->angle1Spx->value() << ui->angle2Spx->value();
        errCode = func(offsetXs,offsetYs,angles);
    }

    if(errCode != MARK_NO_ERROR) {
        QMessageBox::information(this,tr("Error"),tr("Start mark error(%1).").arg(errCode),QMessageBox::Close);
    }
}

